#!/usr/bin/bash

OUTDIR=$1

DURATION_WARMUP=7200
DURATION=300
RUNS=3
DBNAME=test
SCALES="500"
CLIENTS="1 16 32 64 96"
CONFIG='postgresql-12.conf'
WALSEGSIZE="16 512 1024"

echo data scale init vacuum checkpoint tps > $OUTDIR/warmup.csv
echo data scale type run clients tps > $OUTDIR/results.csv
echo data initdb start > $OUTDIR/initdb.csv
echo data stop > $OUTDIR/stop.csv

# data on nvme, wal on pmem
for data in nvme; do

	# basic system info
	mount > $OUTDIR/mount.log
	pg_config > $OUTDIR/pg_config.log

	for ws in $WALSEGSIZE; do

		mkdir $OUTDIR/$data-$ws

		# make sure nothing is running (a bit ugly, but ...)
		killall postgres

		# remove existing directories
		rm -Rf /opt/$data/benchmarks/data
		rm -Rf /opt/pmemdax/benchmarks/wal

		# create empty directories
		mkdir /opt/$data/benchmarks/data
		mkdir /opt/pmemdax/benchmarks/wal

		# initialize the cluster with a WAL on the nvme device
		echo `date` initializing cluster
		initdb_start=`date +%s`
		initdb -X /opt/pmemdax/benchmarks/wal -D /opt/$data/benchmarks/data --wal-segsize=$ws > $OUTDIR/$data-$ws/initdb.log 2>&1
		initdb_end=`date +%s`

		# install the config file
		cp config/$CONFIG /opt/$data/benchmarks/data/postgresql.conf

		# start the cluster
		echo `date` starting cluster
		start_start=`date +%s`
		pg_ctl -D /opt/$data/benchmarks/data -l $OUTDIR/$data-$ws/pg.log -w -t 3600 start > $OUTDIR/$data-$ws/start.log 2>&1
		start_end=`date +%s`

		# dump all the settings
		psql postgres -c 'select * from pg_settings' > $OUTDIR/$data-$ws/pg_settings.log 2>&1

		echo $data $ws $((initdb_end-initdb_start)) $((start_end-start_start)) >> $OUTDIR/initdb.csv 2>&1

		# run benchmarks on all the scales
		for scale in $SCALES; do

			mkdir $OUTDIR/$data-$ws/$scale

			echo `date` recreating database
			dropdb --if-exists $DBNAME >> $OUTDIR/$data-$ws/$scale/init.log 2>&1
			createdb $DBNAME >> $OUTDIR/$data-$ws/$scale/init.log 2>&1

			echo `date` initializing pgbench scale $scale
			init_start=`date +%s`
			pgbench -i -s $scale $DBNAME >> $OUTDIR/$data-$ws/$scale/init.log 2>&1
			init_end=`date +%s`

			echo `date` vacuum
			vacuum_start=`date +%s`
			psql $DBNAME -c "vacuum analyze" >> $OUTDIR/$data-$ws/$scale/init.log 2>&1
			vacuum_end=`date +%s`

			echo `date` checkpoint
			checkpoint_start=`date +%s`
			psql $DBNAME -c "checkpoint" >> $OUTDIR/$data-$ws/$scale/init.log 2>&1
			checkpoint_end=`date +%s`

			# warmup
			echo `date` warmup
			mkdir $OUTDIR/$data-$ws/$scale/warmup
			pgbench -N -j 16 -c 16 -M prepared -T $DURATION_WARMUP -l --sampling-rate=0.05 $DBNAME > $OUTDIR/$data-$ws/$scale/warmup/pgbench.log 2>&1
			mv pgbench_log.* $OUTDIR/$data-$ws/$scale/warmup/

			tps=`grep excluding $OUTDIR/$data-$ws/$scale/warmup/pgbench.log | awk '{print $3}'`

			echo $data $ws $scale $((init_end-init_start)) $((vacuum_end-vacuum_start)) $((checkpoint_end-checkpoint_start)) $tps >> $OUTDIR/$data-$ws/warmup.csv 2>&1

			# read-write
			for r in `seq 1 $RUNS`; do

				for clients in $CLIENTS; do

					mkdir -p $OUTDIR/$data-$ws/$scale/rw/$clients/$r

					# how many threads to use (same as clients, but not more than 16)
					jobs=$clients
					if [ $jobs -gt 16 ]; then
						jobs=16
					fi

					psql $DBNAME -c 'checkpoint' > /dev/null 2>&1

					echo `date` running pgbench r/w data $data segmend $ws scale $scale clients $clients jobs $jobs run $r
					pgbench -N -c $clients -j $jobs -M prepared -T $DURATION -l --sampling-rate=0.05 $DBNAME > $OUTDIR/$data-$ws/$scale/rw/$clients/$r/pgbench.log 2>&1

					mv pgbench_log.* $OUTDIR/$data-$ws/$scale/rw/$clients/$r/

					tps=`grep excluding $OUTDIR/$data-$ws/$scale/rw/$clients/$r/pgbench.log | awk '{print $3}'`

					echo $data $ws $scale rw $r $clients $tps >> $OUTDIR/results.csv 2>&1

				done

			done

		done

		# stop the cluster
		echo `date` stopping cluster
		stop_start=`date +%s`
		pg_ctl -D /opt/$data/benchmarks/data -l $OUTDIR/$data-$ws/pg.log -w -t 3600 stop > $OUTDIR/$data-$ws/stop.log 2>&1
		stop_end=`date +%s`

		echo $data $ws $((stop_end-start_end)) > $OUTDIR/stop.csv

		mkdir $OUTDIR/$data-$ws/sar

		echo `date` exporting sar data
		mkdir `pwd`/$OUTDIR/$data-$ws/sar
		./export-sar.sh `pwd`/$OUTDIR/$data-$ws/sar

	done

done

